@@ -40,7 +40,7 @@ cr_font_family_to_string_real (CRFontFamily * a_this,
         }
 
         if (!a_this) {
-                g_string_append_printf (*a_string, "NULL");
+                g_string_append (*a_string, "NULL");
                 return CR_OK;
         }
 
@@ -96,28 +96,28 @@ cr_predefined_absolute_font_size_to_string (enum CRPredefinedAbsoluteFontSize
 
         switch (a_code) {
         case FONT_SIZE_XX_SMALL:
-                str = (gchar *) "font-size-xx-small";
+                str = (gchar *) "xx-small";
                 break;
         case FONT_SIZE_X_SMALL:
-                str = (gchar *) "font-size-x-small";
+                str = (gchar *) "x-small";
                 break;
         case FONT_SIZE_SMALL:
-                str = (gchar *) "font-size-small";
+                str = (gchar *) "small";
                 break;
         case FONT_SIZE_MEDIUM:
-                str = (gchar *) "font-size-medium";
+                str = (gchar *) "medium";
                 break;
         case FONT_SIZE_LARGE:
-                str = (gchar *) "font-size-large";
+                str = (gchar *) "large";
                 break;
         case FONT_SIZE_X_LARGE:
-                str = (gchar *) "font-size-x-large";
+                str = (gchar *) "x-large";
                 break;
         case FONT_SIZE_XX_LARGE:
-                str = (gchar *) "font-size-xx-large";
+                str = (gchar *) "xx-large";
                 break;
         default:
-                str = (gchar *) "unknown predefined absolute font size value";
+                str = (gchar *) "unknown absolute font size value";
         }
         return str;
 }
@@ -129,10 +129,10 @@ cr_relative_font_size_to_string (enum CRRelativeFontSize a_code)
 
         switch (a_code) {
         case FONT_SIZE_LARGER:
-                str = (gchar *) "font-size-larger";
+                str = (gchar *) "larger";
                 break;
         case FONT_SIZE_SMALLER:
-                str = (gchar *) "font-size-smaller";
+                str = (gchar *) "smaller";
                 break;
         default:
                 str = (gchar *) "unknown relative font size value";
@@ -309,9 +309,6 @@ cr_font_size_clear (CRFontSize * a_this)
                 break;
 
         case ABSOLUTE_FONT_SIZE:
-                if (a_this->value.absolute) {
-                        cr_num_destroy (a_this->value.absolute);
-                }
                 memset (a_this, 0, sizeof (CRFontSize));
                 break;
 
@@ -336,15 +333,10 @@ cr_font_size_copy (CRFontSize * a_dst, CRFontSize * a_src)
                 break;
 
         case ABSOLUTE_FONT_SIZE:
-                if (a_src->value.absolute) {
-                        cr_font_size_clear (a_dst);
-                        if (!a_dst->value.absolute) {
-                                a_dst->value.absolute = cr_num_new ();
-                        }
-                        cr_num_copy (a_dst->value.absolute,
-                                     a_src->value.absolute);
-                        a_dst->type = a_src->type;
-                }
+                cr_font_size_clear (a_dst);
+                cr_num_copy (&a_dst->value.absolute,
+                             &a_src->value.absolute);
+                a_dst->type = a_src->type;
                 break;
 
         default:
@@ -353,6 +345,70 @@ cr_font_size_copy (CRFontSize * a_dst, CRFontSize * a_src)
         return CR_OK;
 }
 
+enum CRStatus 
+cr_font_size_set_predefined_absolute_font_size (CRFontSize *a_this, 
+                                                enum CRPredefinedAbsoluteFontSize a_predefined)
+{
+        g_return_val_if_fail (a_this, CR_BAD_PARAM_ERROR) ;
+        g_return_val_if_fail (a_predefined >= PREDEFINED_ABSOLUTE_FONT_SIZE
+                              && a_predefined < NB_FONT_SIZE_TYPE,
+                              CR_BAD_PARAM_ERROR) ;
+
+        a_this->type = PREDEFINED_ABSOLUTE_FONT_SIZE ;
+        a_this->value.predefined = a_predefined ;
+
+        return CR_OK ;
+}
+
+enum CRStatus 
+cr_font_size_set_relative_font_size (CRFontSize *a_this,
+                                     enum CRRelativeFontSize a_relative)
+{
+        g_return_val_if_fail (a_this, CR_BAD_PARAM_ERROR) ;
+        g_return_val_if_fail (a_relative >= FONT_SIZE_LARGER
+                              && a_relative < NB_RELATIVE_FONT_SIZE,
+                              CR_BAD_PARAM_ERROR) ;
+        
+        a_this->type = RELATIVE_FONT_SIZE ;
+        a_this->value.relative = a_relative ;
+        return CR_OK ;
+}
+
+enum CRStatus 
+cr_font_size_set_absolute_font_size (CRFontSize *a_this,
+                                     enum CRNumType a_num_type,
+                                     gdouble a_value)
+{
+        g_return_val_if_fail (a_this, CR_BAD_PARAM_ERROR) ;
+        g_return_val_if_fail (a_num_type >= NUM_AUTO
+                              && a_num_type < NB_NUM_TYPE,
+                              CR_BAD_PARAM_ERROR) ;
+
+        a_this->type = ABSOLUTE_FONT_SIZE ;
+        cr_num_set (&a_this->value.absolute,
+                    a_value, a_num_type) ;        
+        return CR_OK ;
+}
+
+enum CRStatus
+cr_font_size_set_to_inherit (CRFontSize *a_this)
+{
+        g_return_val_if_fail (a_this, CR_BAD_PARAM_ERROR) ;
+
+        cr_font_size_clear (a_this) ;
+        a_this->type = INHERITED_FONT_SIZE ;
+
+        return CR_OK ;
+}
+
+gboolean
+cr_font_size_is_set_to_inherit (CRFontSize *a_this)
+{
+        g_return_val_if_fail (a_this, FALSE) ;
+
+        return a_this->type == INHERITED_FONT_SIZE ;
+}
+
 gchar *
 cr_font_size_to_string (CRFontSize * a_this)
 {
@@ -369,14 +425,14 @@ cr_font_size_to_string (CRFontSize * a_this)
                                 (a_this->value.predefined));
                 break;
         case ABSOLUTE_FONT_SIZE:
-                str = cr_num_to_string (a_this->value.absolute);
+                str = cr_num_to_string (&a_this->value.absolute);
                 break;
         case RELATIVE_FONT_SIZE:
                 str = g_strdup (cr_relative_font_size_to_string
                                 (a_this->value.relative));
                 break;
         case INHERITED_FONT_SIZE:
-                str = g_strdup ("inherited");
+                str = g_strdup ("inherit");
                 break;
         default:
                 break;
@@ -384,6 +440,106 @@ cr_font_size_to_string (CRFontSize * a_this)
         return str;
 }
 
+void 
+cr_font_size_get_smaller_predefined_font_size (enum CRPredefinedAbsoluteFontSize a_font_size,
+                                               enum CRPredefinedAbsoluteFontSize *a_smaller_size)
+{
+        enum CRPredefinedAbsoluteFontSize result = FONT_SIZE_MEDIUM ;
+
+        g_return_if_fail (a_smaller_size) ;
+        g_return_if_fail (a_font_size < NB_PREDEFINED_ABSOLUTE_FONT_SIZES
+                          && a_font_size >= FONT_SIZE_XX_SMALL) ;
+
+        switch (a_font_size) {
+        case FONT_SIZE_XX_SMALL:
+                result =  FONT_SIZE_XX_SMALL ;
+                break ;
+        case FONT_SIZE_X_SMALL:
+                result =  FONT_SIZE_XX_SMALL ;
+                break ;
+        case FONT_SIZE_SMALL:
+                result =  FONT_SIZE_X_SMALL;
+                break ;
+        case FONT_SIZE_MEDIUM:
+                result =  FONT_SIZE_SMALL;
+                break ;
+        case FONT_SIZE_LARGE:
+                result =  FONT_SIZE_MEDIUM;
+                break ;
+        case FONT_SIZE_X_LARGE:
+                result =  FONT_SIZE_LARGE;
+                break ;
+        case FONT_SIZE_XX_LARGE:
+                result =  FONT_SIZE_XX_LARGE;
+                break ;
+	case FONT_SIZE_INHERIT:
+                cr_utils_trace_info ("can't return a smaller size for FONT_SIZE_INHERIT") ;                
+                result =  FONT_SIZE_MEDIUM ;
+                break ;
+        default:
+                cr_utils_trace_info ("Unknown FONT_SIZE") ;
+                result = FONT_SIZE_MEDIUM ;
+                break ;
+        }
+        *a_smaller_size = result ;
+}
+
+
+void 
+cr_font_size_get_larger_predefined_font_size (enum CRPredefinedAbsoluteFontSize a_font_size,
+                                              enum CRPredefinedAbsoluteFontSize *a_larger_size)
+{
+        enum CRPredefinedAbsoluteFontSize result = FONT_SIZE_MEDIUM ;
+        
+        g_return_if_fail (a_larger_size) ;
+        g_return_if_fail (a_font_size >= FONT_SIZE_XX_SMALL 
+                          && a_font_size < NB_PREDEFINED_ABSOLUTE_FONT_SIZES) ;
+
+        switch (a_font_size) {
+        case FONT_SIZE_XX_SMALL:
+                result =  FONT_SIZE_X_SMALL ;
+                break ;
+        case FONT_SIZE_X_SMALL:
+                result =  FONT_SIZE_SMALL ;
+                break ;
+        case FONT_SIZE_SMALL:
+                result =  FONT_SIZE_MEDIUM;
+                break ;
+        case FONT_SIZE_MEDIUM:
+                result =  FONT_SIZE_LARGE;
+                break ;
+        case FONT_SIZE_LARGE:
+                result =  FONT_SIZE_X_LARGE;
+                break ;
+        case FONT_SIZE_X_LARGE:
+                result =  FONT_SIZE_XX_LARGE ;
+                break ;
+        case FONT_SIZE_XX_LARGE:
+                result =  FONT_SIZE_XX_LARGE;
+                break ;
+	case FONT_SIZE_INHERIT:
+                cr_utils_trace_info ("can't return a bigger size for FONT_SIZE_INHERIT") ;                
+                result =  FONT_SIZE_MEDIUM ;
+                break ;
+        default:
+                cr_utils_trace_info ("Unknown FONT_SIZE") ;
+                result = FONT_SIZE_MEDIUM ;
+                break ;
+        }
+        *a_larger_size = result ;
+}
+
+gboolean
+cr_font_size_is_predefined_absolute_font_size (enum CRPredefinedAbsoluteFontSize a_font_size)
+{
+        if (a_font_size >= FONT_SIZE_XX_SMALL
+            && a_font_size < NB_PREDEFINED_ABSOLUTE_FONT_SIZES) {
+                return TRUE ;
+        } else {
+                return FALSE ;
+        }
+}
+
 gchar *
 cr_font_size_adjust_to_string (CRFontSizeAdjust * a_this)
 {
@@ -397,16 +553,16 @@ cr_font_size_adjust_to_string (CRFontSizeAdjust * a_this)
 
         switch (a_this->type) {
         case FONT_SIZE_ADJUST_NONE:
-                str = g_strdup ("font-size-adjust-none");
+                str = g_strdup ("none");
                 break;
         case FONT_SIZE_ADJUST_NUMBER:
                 if (a_this->num)
                         str = cr_num_to_string (a_this->num);
                 else
-                        str = g_strdup ("font-size-adjust-NULL");
+                        str = g_strdup ("unknow font-size-adjust property value"); // Should raise an error no?
                 break;
         case FONT_SIZE_ADJUST_INHERIT:
-                str = g_strdup ("font-size-adjust-inherit");
+                str = g_strdup ("inherit");
         }
         return str;
 }
@@ -418,19 +574,19 @@ cr_font_style_to_string (enum CRFontStyle a_code)
 
         switch (a_code) {
         case FONT_STYLE_NORMAL:
-                str = (gchar *) "font-style-normal";
+                str = (gchar *) "normal";
                 break;
         case FONT_STYLE_ITALIC:
-                str = (gchar *) "font-style-italic";
+                str = (gchar *) "italic";
                 break;
         case FONT_STYLE_OBLIQUE:
-                str = (gchar *) "font-style-oblique";
+                str = (gchar *) "oblique";
                 break;
         case FONT_STYLE_INHERIT:
-                str = (gchar *) "font-style-inherit";
+                str = (gchar *) "inherit";
                 break;
         default:
-                str = (gchar *) "font-style";
+                str = (gchar *) "unknown font style value";
                 break;
         }
         return str;
@@ -443,18 +599,34 @@ cr_font_variant_to_string (enum CRFontVariant a_code)
 
         switch (a_code) {
         case FONT_VARIANT_NORMAL:
-                str = (gchar *) "font-variant-normal";
+                str = (gchar *) "normal";
                 break;
         case FONT_VARIANT_SMALL_CAPS:
-                str = (gchar *) "font-variant-small-caps";
+                str = (gchar *) "small-caps";
                 break;
         case FONT_VARIANT_INHERIT:
-                str = (gchar *) "font-variant-inherent";
+                str = (gchar *) "inherit";
                 break;
         }
         return str;
 }
 
+enum CRFontWeight
+cr_font_weight_get_bolder (enum CRFontWeight a_weight)
+{
+        if (a_weight >= NB_FONT_WEIGHTS) {
+                return FONT_WEIGHT_900 ;
+        } else if (a_weight < FONT_WEIGHT_NORMAL) {
+                return FONT_WEIGHT_NORMAL ;
+        } else if (a_weight == FONT_WEIGHT_BOLDER
+                   || a_weight == FONT_WEIGHT_BOLDER) {
+                cr_utils_trace_info ("FONT_WEIGHT_BOLDER or FONT_WEIGHT_LIGHTER should not appear here") ;
+                return FONT_WEIGHT_NORMAL ;
+        } else {
+                return a_weight << 1 ;
+        }
+}
+
 const gchar *
 cr_font_weight_to_string (enum CRFontWeight a_code)
 {
@@ -462,46 +634,46 @@ cr_font_weight_to_string (enum CRFontWeight a_code)
 
         switch (a_code) {
         case FONT_WEIGHT_NORMAL:
-                str = (gchar *) "font-weight-normal";
+                str = (gchar *) "normal";
                 break;
         case FONT_WEIGHT_BOLD:
-                str = (gchar *) "font-weight-bold";
+                str = (gchar *) "bold";
                 break;
         case FONT_WEIGHT_BOLDER:
-                str = (gchar *) "font-weight-bolder";
+                str = (gchar *) "bolder";
                 break;
         case FONT_WEIGHT_LIGHTER:
-                str = (gchar *) "font-weight-lighter";
+                str = (gchar *) "lighter";
                 break;
         case FONT_WEIGHT_100:
-                str = (gchar *) "font-weight-100";
+                str = (gchar *) "100";
                 break;
         case FONT_WEIGHT_200:
-                str = (gchar *) "font-weight-200";
+                str = (gchar *) "200";
                 break;
         case FONT_WEIGHT_300:
-                str = (gchar *) "font-weight-300";
+                str = (gchar *) "300";
                 break;
         case FONT_WEIGHT_400:
-                str = (gchar *) "font-weight-400";
+                str = (gchar *) "400";
                 break;
         case FONT_WEIGHT_500:
-                str = (gchar *) "font-weight-500";
+                str = (gchar *) "500";
                 break;
         case FONT_WEIGHT_600:
-                str = (gchar *) "font-weight-600";
+                str = (gchar *) "600";
                 break;
         case FONT_WEIGHT_700:
-                str = (gchar *) "font-weight-700";
+                str = (gchar *) "700";
                 break;
         case FONT_WEIGHT_800:
-                str = (gchar *) "font-weight-800";
+                str = (gchar *) "800";
                 break;
         case FONT_WEIGHT_900:
-                str = (gchar *) "font-weight-900";
+                str = (gchar *) "900";
                 break;
         case FONT_WEIGHT_INHERIT:
-                str = (gchar *) "font-weight-inherit";
+                str = (gchar *) "inherit";
                 break;
         default:
                 str = (gchar *) "unknown font-weight property value";
@@ -517,40 +689,40 @@ cr_font_stretch_to_string (enum CRFontStretch a_code)
 
         switch (a_code) {
         case FONT_STRETCH_NORMAL:
-                str = (gchar *) "font-stretch-normal";
+                str = (gchar *) "normal";
                 break;
         case FONT_STRETCH_WIDER:
-                str = (gchar *) "font-stretch-wider";
+                str = (gchar *) "wider";
                 break;
         case FONT_STRETCH_NARROWER:
-                str = (gchar *) "font-stretch-narrower";
+                str = (gchar *) "narrower";
                 break;
         case FONT_STRETCH_ULTRA_CONDENSED:
-                str = (gchar *) "font-stretch-ultra-condensed";
+                str = (gchar *) "ultra-condensed";
                 break;
         case FONT_STRETCH_EXTRA_CONDENSED:
-                str = (gchar *) "font-stretch-extra-condensed";
+                str = (gchar *) "extra-condensed";
                 break;
         case FONT_STRETCH_CONDENSED:
-                str = (gchar *) "font-stretch-condensed";
+                str = (gchar *) "condensed";
                 break;
         case FONT_STRETCH_SEMI_CONDENSED:
-                str = (gchar *) "font-stretch-semi-condensed";
+                str = (gchar *) "semi-condensed";
                 break;
         case FONT_STRETCH_SEMI_EXPANDED:
-                str = (gchar *) "font-stretch-semi-expanded";
+                str = (gchar *) "semi-expanded";
                 break;
         case FONT_STRETCH_EXPANDED:
-                str = (gchar *) "font-stretch-expanded";
+                str = (gchar *) "expanded";
                 break;
         case FONT_STRETCH_EXTRA_EXPANDED:
-                str = (gchar *) "font-stretch-extra-expaned";
+                str = (gchar *) "extra-expaned";
                 break;
         case FONT_STRETCH_ULTRA_EXPANDED:
-                str = (gchar *) "font-stretch-ultra-expanded";
+                str = (gchar *) "ultra-expanded";
                 break;
         case FONT_STRETCH_INHERIT:
-                str = (gchar *) "font-stretch-inherit";
+                str = (gchar *) "inherit";
                 break;
         }
         return str;
@@ -561,11 +733,7 @@ cr_font_size_destroy (CRFontSize * a_font_size)
 {
         g_return_if_fail (a_font_size);
 
-        if (a_font_size->type == ABSOLUTE_FONT_SIZE
-            && a_font_size->value.absolute) {
-                cr_num_destroy (a_font_size->value.absolute);
-                a_font_size->value.absolute = NULL;
-        }
+        g_free (a_font_size) ;
 }
 
 /*******************************************************
