@@ -1,5 +1,5 @@
 TEST_INPUTS_DIR=$HERE_DIR/test-inputs
-CSSLINT=$HERE_DIR/../csslint/.libs/csslint
+CSSLINT=$HERE_DIR/../csslint/.libs/csslint-0.6
 if ! test -x $CSSLINT ; then
     echo "Aarg Could not find an executable csslint. I was looking for $CSSLINT"
     echo $
